"""
Testing suite for wf_monitor. Currently not covered by tests:
- log_decorator_factory
- make_arg_parser
"""

import logging
import os
import sys
import threading
import time
from typing import Dict
from unittest.mock import patch

import pytest
import wf_monitor.monitor
from _pytest.logging import LogCaptureFixture
from pyramid.router import Router
from wf_monitor.monitor import WorkflowMonitor

log_path = "logs/condor.log"

logger = logging.getLogger("test_wf_monitor")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def mock_send_message(message: Dict[str, str], mock_router: Router):
    """
    Mocked version of wf_monitor.monitor.send_event without any AMQP connections being made

    :param message: Workflow event being "sent"
    :param mock_router: Mock router
    """
    logger.info(f"Pretending to send message: {message} from mock router: {mock_router}")


wf_monitor.monitor.send_message = mock_send_message
# class MockWorkflowMonitor(WorkflowMonitor):
#     """
#     Class that mocks AMQP portions of WorkflowMonitor's functionality
#     """
#
#     def read_htcondor_log(self, timeout: int):
#         exists = False
#         try:
#             exists = self.log_exists(timeout)
#         except TimeoutError as e:
#             # log_exists() timed out
#             logger.error(e)
#             quit(-1)
#         if exists:
#             logger.info("Log file found.")
#             event_str = ""
#             with open(self.logfile_path, "r") as f:
#                 # Read while file is being written to
#                 while True:
#                     if self.num_jobs == 0:
#                         return
#
#                     where = f.tell()
#                     line = f.readline()
#                     if not line:
#                         time.sleep(1)
#                         f.seek(where)
#                     else:
#                         if line == "...\n":
#                             # Line is at the end of an event
#                             event_str += line
#                             # Parse and send event
#                             try:
#                                 event = self.parse_message(event_str)
#                             except ValueError as e:
#                                 # Event not well-formed
#                                 logger.error(e)
#                                 continue
#                             mock_send_event(event)
#                             # Empty contents to prepare for next event
#                             self.log += event_str
#                             event_str = ""
#                         elif line == "\n":
#                             # Line empty
#                             time.sleep(1)
#                         else:
#                             # Line is anywhere else
#                             event_str += line
#
#
# test_monitor = MockWorkflowMonitor(log_path, -1)


def mock_monitor_factory(path_to_log: str, timeout: int = 60) -> WorkflowMonitor:
    """
    Factory function that creates a WorkflowMonitor object that has its external dependencies mocked away

    :param path_to_log: Path to log file that will be monitored
    :return: Mocked WorkflowMonitor
    """
    with patch("wf_monitor.monitor.Router"):
        return WorkflowMonitor(path_to_log, -1, timeout)


default_mock_monitor = mock_monitor_factory(log_path)


def test_read_log():
    """
    Tests whether or not the example log is being correctly read by checking for strings known to
    exist in the example log file
    """
    test_strs = [
        "Image size of job updated: 72",
        "72  -  ResidentSetSize of job (KB)",
        "(1) Normal termination (return value 0)",
        "000 (3983.000.000) 08/26 11:06:06 Job submitted from host: "
        "<10.64.1.178:9618?addrs=10.64.1.178-9618&noUDP&sock=4050180_7e37_3>",
    ]
    for s in test_strs:
        assert s in default_mock_monitor.log


@pytest.mark.skip(reason="This test sometimes fails. Skipping until it is investigated further.")
def test_read_log_slow(caplog: LogCaptureFixture):
    """
    Tests that wf_monitor can keep up with reading when writing is slow.
    """
    caplog.set_level(logging.INFO)
    slow_log = "logs/slow.log"
    print(os.getcwd())
    # FIXME: This line fails
    thread = threading.Thread(target=mock_monitor_factory, args=slow_log)
    thread.start()
    # Truncate slow.log
    open(slow_log, "w").close()
    event = ""
    with open(slow_log, "r") as log:
        for line in log:
            print(line)
            event += line
            if line == "...\n":
                # End of event reached
                with open(slow_log, "a") as slow:
                    slow.write(event)
                    event = ""
                    time.sleep(2)

    thread.join()
    assert f"Finished monitoring {slow_log}.log." in caplog.text


def test_read_log_timeout():
    """
    Tests the timeout functionality of WorkflowMonitor.read_htcondor_log()
    """
    with pytest.raises(SystemExit) as sys_ex:
        mock_monitor_factory("logs/file-that-does-not-exist.txt", 1)
    assert sys_ex.type == SystemExit
    assert sys_ex.value.code == -1


def test_parse_log():
    """
    Tests that the example log file is correctly parsed by checking for the known order of event
    types
    """
    test_event_types = [
        "workflow-executing",
        "unknown",
        "unknown",
        "workflow-complete",
    ]
    for msg, event_type in zip(default_mock_monitor.messages, test_event_types):
        assert msg["type"] == event_type


def test_parse_log_error(caplog: LogCaptureFixture):
    """
    Tests that WorkflowMonitor.parse_log() correctly raises a ValueError when a badly formatted
    log is parsed
    """
    caplog.set_level(logging.INFO)
    with pytest.raises(TimeoutError):
        mock_monitor_factory("logs/test.log", 5)
    msg = (
        "HTCondor event not well-formatted. Event contents:\n"
        "this will trigger an exception!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n...\n"
    )
    messages = [record.message for record in caplog.records]
    assert msg in messages


def test_send_events(caplog: LogCaptureFixture):
    """
    Test that mocks the core wf_monitor functionality of creating a connection to the RabbitMQ
    server, configuring a channel with that connection, and emitting events through that channel
    """
    caplog.set_level(logging.INFO)
    for message in default_mock_monitor.messages:
        mock_send_message(message, default_mock_monitor.message_router)
    messages = [record.message for record in caplog.records]
    for msg in messages:
        assert msg.startswith("Pretending to send message")
